/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003-2004  Ciaran Anscomb
 *
 *  See COPYING for redistribution conditions. */

#ifndef __PIA_H__
#define __PIA_H__

#include "keyboard.h"
#include "joystick.h"
#include "sound.h"
#include "vdg.h"
#include "types.h"
#include "m6809.h"

#define MAX_WRFUNCS 4

//typedef void (*write_function)(void);
typedef struct {
	Uint32 control_register;
	Uint32 direction_register;
	Uint32 output_register;
	Uint32 port_output;
	Uint32 port_input;
	Uint32 tied_low;
	/* Convenience flags split out from control_register above */
	Uint32 interrupt_enable;
	Uint32 interrupt_transition;
	Uint32 register_select;  /* 0 = DDR, SET = OR */
	/* ignore Cx2 stuff for now */
	//write_function wrfunc[MAX_WRFUNCS];
} pia_port;

extern pia_port PIA_0A, PIA_0B, PIA_1A, PIA_1B;

#define PIA_SET_P0CA1 PIA_0A.control_register |= 0x80; if (PIA_0A.interrupt_enable) irq = 1;
#define PIA_SET_P0CB1 PIA_0B.control_register |= 0x80; if (PIA_0B.interrupt_enable) irq = 1;
#define PIA_SET_P1CA1 PIA_1A.control_register |= 0x80; if (PIA_1A.interrupt_enable) firq = 1;
#define PIA_SET_P1CB1 PIA_1B.control_register |= 0x80; if (PIA_1B.interrupt_enable) firq = 1;

#define PIA_WRITE(p,v) if (!p.register_select) { \
		p.direction_register = v; \
		v &= p.output_register; \
	} else { \
		p.output_register = v; \
		v &= p.direction_register; \
	} \
	p.port_output = (v | (p.port_input & ~(p.direction_register))) & p.tied_low

#define PIA_WRITE_P0DA(v) { PIA_WRITE(PIA_0A, v); keyboard_row_update(); }
#define PIA_WRITE_P0DB(v) { PIA_WRITE(PIA_0B, v); keyboard_column_update(); }
#define PIA_WRITE_P1DA(v) { PIA_WRITE(PIA_1A, v); sound_update(); joystick_update(); }
#define PIA_WRITE_P1DB(v) { PIA_WRITE(PIA_1B, v); vdg_set_mode(); }

void pia_init(void);
void pia_reset(void);
void pia_control_write(pia_port *p, Uint32 octet);
Uint32 pia_control_read(pia_port *p);
Uint32 pia_data_read(pia_port *p);
/* This is called by devices to set input bits */
void pia_data_set(pia_port *p, Uint32 octet);

#endif  /* __PIA_H__ */
